#ifndef DRIVE_H
#define DRIVE_H

namespace iRobot
{

class Robot;

/**
 * Controls the robot's drive wheels.
 *
 * The iRobot Create has two drive wheels, both with a maximum speed of 
 * 500mm/s. When using the move() method, you have a choice of turn radius 
 * ranging from 0 to 2000mm.
 *
 * Keep in mind, that the robot will not move in precicely the speed or turn
 * radius desired due to mechanics such as wheel slipping.
 */
class Drive
{
    public:
		/**
		 * Guide values for speeds
		 */
        typedef enum {
			/**
			 * 100mm/s
			 */
            Slow = 100,
			/**
			 * 250mm/s. Half of maximum speed.
			 */
            Medium = 250,
			/**
			 * 500mm/s. Maximum speed.
			 */
            Fast = 500
        } Speed;

		/**
		 * These special values allow the robot to have a zero turn radius.
		 */
        typedef enum {
			/**
			 * Turn clockwise on the spot.
			 */
            Right = 0xFFFF,
			/**
			 * No turning, just drive straight.
			 */
            Straight = 0x7FFF,
			/**
			 * Turn counter-clockwise on the spot.
			 */
            Left = 0x0001
        } TurnDirection;

		/**
		 * Constructor.
		 * 
		 * @param robot The robot to which the Drive is attached.
		 * @see Robot::getDrive()
		 */
        Drive ( Robot* robot );
		
		/**
		 * Destructor.
		 */
        ~Drive();

		void turnWheels( int leftWheel, int rightWheel );

		/**
		 * Turn with the given speed, turning in the given direction for the
         * given distance in mm
		 */
        void move ( Speed speed, TurnDirection direction = Straight );
		/**
		 * Turn with the given speed (in mm/s), turning in the given direction.
		 */
        void move ( int speed, TurnDirection direction = Straight );
		/**
		 * Move with the given speed in a circle with the given radius (in mm)
		 */
        void move ( Speed speed, int radius );
		/**
		 * Move with the given speed (in mm/s) in a circle with the given radius
		 * (in mm). A positive radius turns left, while a negative one turns
		 * right.
		 *
		 * @param speed Speed, in mm/s.
		 * @param radius Turn radius, in mm/s. Positive is left, negative is
		 * right. Zero is straight ahead.
		 */
        void move ( int speed, int radius );
        
        /**
         * Waits until the robot has turned the number of degrees.
         * Returns the total (not net) angle turned.
         */
        int waitForRotation( int angle );
        
        /**
         * Waits until the robot has moved the number of mm
         * Returns the total (not net) distance traveled.
         */
        int waitForDistance( int distance );
    private:
        Robot* m_robot;
};

}

#endif
