#ifndef ROBOT_H
#define ROBOT_H

#include <libirobot/Sensor.h>
#include <libirobot/Song.h>

namespace iRobot
{

class SerialControl;
class Drive;

namespace Private {
class SerialControl;
}

/**
 *  The main interface to control an iRobot.
 *
 *  The Robot class represents a single iRobot. It is the main method of
 *  creating a program to control a single iRobot. The 
 */

class Robot
{
    public:
		/**
		 * Mode of control.
		 *
		 * According to the Open Interface description, there are three modes: Passive, Safe, and Full.
		 *
		 * Passive mode...
		 *
		 * Safe mode gives you complete access, but keeps some failsafes enabled. In particular, the cliff and drop sensors.
		 * When those sensors are triggered, the iRobot reverts back to Passive mode.
		 *
		 * Full gives you complete access and disables all failsafes. This means
		 * if you tell the robot to drive off the cliff, it'll happily oblige :)
		 */
        typedef enum {
            Off,
            Passive,
            Safe,
            Full
        } Mode;

		/**
		 * Constructor.
		 *
		 * Creates a new Robot.
		 *
		 * @param ttyDevice Path to the serial device
		 */
        Robot ( const char ttyDevice[] );

		/**
		 * Destructor.
		 */
        ~Robot();

		/**
		 * Returns the robot's drive control.
		 *
		 * @see Drive
		 */
        Drive* getDrive() const;

		/**
		 * Returns the underlying serial connection.
		 *
		 * @see SerialControl
		 */
        SerialControl* getSerial() const;

		/**
		 * Returns an instance of the requested sensor.
		 *
		 * Once a sensor is requested, it is updated forever. Until then, it does not exist in memory.
		 *
		 * @param sensor The sensor to request
		 * @see Sensor
		 */
        Sensor* getSensor ( Sensor::SensorID sensor );

		/**
		 * Assigns the song to the given slot
		 * @param songNum Slot to save the song into. Can be from 0-15.
		 * @param song Song to save into the slot
		 * @see playSong()
		 */
		void setSong(uint8_t songNum, const Song &song);

		/**
		 * Starts playing the song.
		 *
		 * Songs are played in the background, allowing the robot to move around
		 * while one is playing.
		 *
		 * @see Sensor::SongPlaying
		 * @see setSong()
		 */
		void playSong(uint8_t songNum);

		typedef enum {
			Play = 2,
			Advance = 8
		} LED;

		void setLEDs(uint8_t leds, uint8_t powerColor, uint8_t powerIntensity);

		/**
		 * Begins running the main loop.
		 *
		 * @see loop()
		 */
        int run();

		/**
		 * The loop.
		 *
		 * Must be implemented by subclasses.
		 *
		 * @see run()
		 */
        virtual void loop() = 0;

		/**
		 * Called when the serial connection is lost.
		 */
        virtual void disconnected();

		/**
		 * Causes run() to exit.
		 * @param ret The return value for run()
		 */
        void exit ( int ret );

        /**
         * Updates every sensor marked for auto-update
         * @see setAutoUpdate()
         */
        
        void updateSensors();
        
		/**
		 * Updates the given sensor
		 */
        void updateSensor( Sensor::SensorID sensor );
        
        /**
         * Adds the given sensor to the list of sensors that get updated at
         * every iteration of loop()
         */
        void setAutoUpdate( Sensor::SensorID sensor, bool enabled );

		/**
		 * Sleeps for the given number of microseconds. Use this instead of
		 * usleep() for cross-platform compatability on the command module.
		 * @param us Microseconds to sleep for
		 */
        void delay ( unsigned int us );

		/**
		 * Sets the iRobot's operating mode.
		 * @see Mode
		 */
        void setMode ( Mode );

#ifdef __AVR__
		/**
		 * Returns the global Robot instance. Only available on the Command
		 * Module.
		 */
        static Robot* instance();
#endif

    private:
        friend class SerialControl;
        friend class Private::SerialControl;
        void sensorUpdated ( Sensor::SensorID sensor );
        bool m_running;
        bool m_exited;
        int m_return;
        SerialControl* m_serial;
        Drive* m_drive;
        Sensor* m_sensors[NUM_SENSORS];
        uint8_t m_sensorValues[52];
        bool m_autoUpdate[NUM_SENSORS];

#ifdef __AVR__
        static Robot* m_instance;
#endif
};

}

#endif
