#ifndef SENSOR_H
#define SENSOR_H

#define NUM_SENSORS 42

#include <libirobot/types.h>

namespace iRobot
{

/**
 * Base class of all sensors on a iRobot.
 *
 * All forms of inputs on an iRobot are available as Sensors. There are
 * different sensors for different inputs, such as unsigned value, signed value,
 * infrared, bumps, etc. For details, check out each specific sensor.
 */

class Sensor
{
    public:
		/**
		 *	Sensor IDs.
		 *	
		 *	Each sensor returns a specific subclass of Sensor. The sensor groups
		 *	refer to specific groups of sensors, and can't be passed to
		 *	Robot::getSensor().
		 *
		 *	@see Robot::getSensor()
		 */
        typedef enum {
            Group0 = 0,
            Group1 = 1,
            Group2 = 2,
            Group3 = 3,
            Group4 = 4,
            Group5 = 5,
			/**
			 * All sensors.
			 */
            All = 6,
			/**
			 * Refers to the BumpAndDropSensor.
			 *
			 * @see BumpAndDropSensor
			 */
            BumpsAndDrops = 7,
			/**
			 *	The wall detector on the right side of the robot. Boolean.
			 *	@see BooleanSensor
			 *	@see WallSignal
			 */
            Wall = 8,
			/**
			 * Far left cliff sensor, under the bumper. Boolean.
			 * @see BooleanSensor
			 * @see CliffLeftSignal
			 */
            CliffLeft = 9,
			/**
			 * Front left cliff sensor, under the bumper. Boolean.
			 * @see BooleanSensor
			 * @see CliffFrontLeftSignal
			 */
            CliffFrontLeft = 10,
			/**
			 * Front right cliff sensor, under the bumper. Boolean.
			 * @see BooleanSensor
			 * @see CliffFrontRightSignal
			 */
            CliffFrontRight = 11,
			/**
			 * Far right cliff sensor, under the bumper. Boolean.
			 * @see BooleanSensor
			 * @see CliffRightSignal
			 */
            CliffRight = 12,
			/**
			 * Boolean sensor for the virtual wall's IR signal.
			 * @see BooleanSensor
			 * @see IRSensor
			 */
            VirtualWall = 13,
			/**
			 * The state of the three low side driver and two wheel overcurrent
			 * sensors.
			 * @see OvercurrentSensor
			 */
            LowDrive = 14,
			/**
			 * The current byte recieved by the IR sensor.
			 * @see IRSensor
			 */
            IR = 17,
            //Two unused sensors here
			/**
			 * Net distance traveled. It gets reset after every request for it.
			 * If not polled frequently enough, it can get capped at it's
			 * maximum or minimum.
			 * @see ValueSensor
			 */
            Distance = 19,
			/**
			 * Net angle turned. It gets reset after every request for it.
			 * Counter-clockwise angles are positive and clockwise angles are
			 * negative. If not polled frequently enough, it can get capped at
			 * it's maximum/minimum.
			 * @see ValueSensor
			 */
            Angle = 20,
			/**
			 * The iRobot's current charging state.
			 * @see ChargeStateSensor
			 * @see ChargeSourceSensor
			 */
            ChargeState = 21,
			/**
			 * Battery voltage, in mV
			 * @see UnsignedValueSensor
			 */
            Voltage = 22,
			/**
			 * Battery current, in mA, flowing into or out of the battery.
			 * Positive indicates power is flowing in, as during charging.
			 * Negative indicates discharge.
			 * @see ValueSensor
			 */
            Current = 23,
			/**
			 * Temperature of the battery in degrees Celsius.
			 * @see UnsignedValueSensor
			 */
            BatteryTemperature = 24,
			/**
			 * Current charge of the battery in mAh. This is not accurate for
			 * the alkaline battery pack.
			 * @see UnsignedValueSensor
			 * @see BatteryCapacity
			 */
            BatteryCharge = 25,
			/**
			 * Estimated capacity of the battery pack in mAh. This is not
			 * accurate for the alkaline battery pack.
			 * @see UnsignedValueSensor
			 * @see BatteryCharge
			 */
            BatteryCapacity = 26,
			/**
			 * Signal strength of the wall proximity sensor.
			 * @see UnsignedValueSensor
			 * @see Wall
			 */
            WallSignal = 27,
			/**
			 * Signal strength as reported by the far left cliff sensor, found
			 * under the front bumper.
			 * @see UnsignedValueSensor
			 * @see CliffLeft
			 */
            CliffLeftSignal = 28,
			/**
			 * Signal strength as reported by the front left cliff sensor, 
			 * found under the front bumper.
			 * @see UnsignedValueSensor
			 * @see CliffFrontLeft
			 */
            CliffFrontLeftSignal = 29,
			/**
			 * Signal strength as reported by the front right cliff sensor,
			 * found under the front bumper.
			 * @see UnsignedValueSensor
			 * @see CliffFrontRight
			 */
            CliffFrontRightSignal = 30,
			/**
			 * Signal strength as reported by the far right cliff sensor, found
			 * under the front bumper.
			 * @see UnsignedValueSensor
			 * @see CliffRight
			 */
            CliffRightSignal = 31,
			/**
			 * Status of the digital inputs in the cargo bay connector.
			 * @see CargoDigitalInputSensor
			 * @see CargoAnalogInput
			 */
            CargoDigitalInputs = 32,
			/**
			 * Status of the analog input in the cargo bay connector.
			 * @see UnsignedValueSensor
			 * @see CargoDigitalInput
			 */
            CargoAnalogInput = 33,
			/**
			 * The charge sources available to the iRobot.
			 * @see ChargeSourceSensor
			 * @see ChargeState
			 */
            ChargeSources = 34,
			/**
			 * The current operating mode of the iRobot
			 * @see Robot::setMode()
			 * @see ModeSensor
			 */
            Mode = 35,
			/**
			 * Numeric ID of song currently selected
			 * @see UnsignedValueSensor
			 * @see Robot::playSong()
			 */
            Song = 36,
			/**
			 * Boolean sensor to determine if a song is playing or not.
			 * @see BooleanSensor
			 * @see Robot::playSong()
			 */
            SongPlaying = 37,
			/**
			 * The number of packets being returned in the stream
			 * @see UnsignedValueSensor
			 */
            StreamPackets = 38,
			/**
			 * Velocity most recently requested with Drive::move()
			 * @see Drive::move()
			 * @see ValueSensor
			 */
            Velocity = 39,
			/**
			 * Turn radius most recently requested with Drive::move()
			 * @see Drive::move()
			 * @see ValueSensor
			 */
            Radius = 40,
			/**
			 * Right wheel's most recently requested velocity from a Drive
			 * Direct command.
			 * TODO: Implement the Drive Direct command
			 * @see ValueSensor
			 */
            RightVelocity = 41,
			/**
			 * Left wheel's most recently requested velocity from a Drive Direct
			 * command.
			 * TODO: Implement the Drive Direct command
			 * @see ValueSensor
			 */
            LeftVelocity = 42,
        } SensorID;

		/**
		 * Factory method to generate a new sensor subclass based on the sensor
		 * requested.
		 * @param sensor Sensor to request
		 */
        static Sensor* get ( SensorID sensor );
		/**
		 * The size of each sensor packet.
		 * FIXME: internal?
		 */
        static int packetSize ( SensorID type );
		/**
		 * In a request for all packets, this returns the location of a given
		 * packet.
		 * FIXME: internal?
		 */
        static int packetOffset ( SensorID type );
    protected:
        Sensor();
        virtual ~Sensor();
        friend class Robot;
		/**
		 * Called from within Robot::sensorUpdated.
		 * @param data An array of bytes, with a length of that returned by
		 * packetSize()
		 */
        virtual void update ( uint8_t data[] ) = 0;
};

}
#endif

