#include "SerialControl.h"

#ifdef __AVR__
#include <avr/interrupt.h>
#include <avr/io.h>
#else
#include <conexus.h>
#endif

using namespace std;

#include "Robot.h"
#include "Sensor.h"
#include <math.h>

using namespace iRobot;

namespace iRobot
{
namespace Private {

class SerialControl
{
    public:
        SerialControl ( Robot* robot, const char ttyDevice[] )
                : m_robot ( robot ) {
#ifndef __AVR__
            Conexus::init();
            m_device = Conexus::TTY::create ( ttyDevice );
#endif
        };
        Robot* m_robot;

#ifdef __AVR__
        Sensor::SensorID sensorID;
        uint8_t curSensor;
        bool waiting;
        void waitForSensors ( Sensor::SensorID id ) {
            sensorID = id;
            curSensor = 0;
            waiting = true;
            while ( waiting );
            m_robot->sensorUpdated ( id );
        }
        void serialPacket ( uint8_t data ) {
            m_robot->m_sensorValues[curSensor+Sensor::packetOffset ( sensorID ) ] = data;
            if ( curSensor == Sensor::packetSize ( sensorID ) )
                waiting = false;
        };
#else
        Conexus::TTY::pointer m_device;
#endif
};

}
}


SerialControl::SerialControl ( Robot* robot, const char ttyDevice[] )
        : m_p ( new Private::SerialControl ( robot, ttyDevice ) )
{
#ifdef __AVR__
    cli();

    //Set I/O pins
    DDRB = 0x10;
    PORTB = 0xCF;
    DDRC = 0x00;
    PORTC = 0xFF;
    DDRD = 0xE6;
    PORTD = 0x7D;

    //Set up serial port with rx interrupt
    UBRR0 = 19;
    UCSR0B = ( _BV ( RXCIE0 ) | _BV ( TXEN0 ) | _BV ( RXEN0 ) );
    UCSR0C = ( _BV ( UCSZ00 ) | _BV ( UCSZ01 ) );

    //start interrupts
    sei();
#else
    m_p->m_device->set_flow_control ( Conexus::FLOW_NONE );
    m_p->m_device->set_speed ( 57600 );
    m_p->m_device->set_stop_bits ( 1 );
    m_p->m_device->set_parity ( Conexus::PARITY_NONE );
#endif
}

SerialControl::~SerialControl()
{
    delete m_p;
}

Robot*
SerialControl::getRobot() const
{
    return m_p->m_robot;
}

void
SerialControl::setBaud ( Baud baud )
{
    write ( SetBaud );

#ifdef __AVR__
    UCSR0A |= _BV ( TXC0 );
#endif

    write ( baud );

#ifdef __AVR__
    while ( ! ( UCSR0A & _BV ( TXC0 ) ) );
    cli();

    UBRR0 = baud;

    /*    switch(baud) {
     case Baud300:
         UBRR0 = Ubrr300;
         break;
     case Baud600:
         UBRR0 = Ubrr600;
         break;
     case Baud1200:
         UBRR0 = Ubrr1200;
         break;
     case Baud2400:
         UBRR0 = Ubrr2400;
         break;
     case Baud4800:
         UBRR0 = Ubrr4800;
         break;
     case Baud9600:
         UBRR0 = Ubrr9600;
         break;
     case Baud14400:
         UBRR0 = Ubrr14400;
         break;
     case Baud19200:
         UBRR0 = Ubrr19200;
         break;
     case Baud28800:
         UBRR0 = Ubrr28800;
         break;
     case Baud38400:
         UBRR0 = Ubrr38400;
         break;
     case Baud57000:
         UBRR0 = Ubrr57000;
         break;
     case Baud115200:
         UBRR0 = Ubrr115200;
         break;
        }*/
    sei();
#else
    m_p->m_device->set_speed ( 300*pow ( 2,baud ) );
#endif
    getRobot()->delay ( 10000 );

}

void
SerialControl::write ( uint8_t byte )
{
#ifdef __AVR__
    while ( ! ( UCSR0A & _BV ( UDRE0 ) ) );
    UDR0 = byte;
#else
    try {
        m_p->m_device->write ( &byte, sizeof ( uint8_t ), -1 );
    } catch ( Conexus::exception::write::timeout e ) {
        m_p->m_robot->disconnected();
    }
#endif
}

void
SerialControl::write ( OpCode op )
{
    write ( ( uint8_t ) op );
}

void
SerialControl::write ( const Song &song )
{
	write(song.getLength());
	for(int i = 0;i < song.getLength();i++) {
		write(song.getNote(i));
	}
}

void
SerialControl::write ( const Song::Note &note )
{
	write(note.pitch);
	write(note.length);
}

#ifdef __AVR__

Private::SerialControl* controlPrivate = 0;

SIGNAL ( SIG_USART_RECV )
{
    uint8_t temp;


    temp = UDR0;

    controlPrivate->serialPacket ( temp );
}
#endif

void
SerialControl::requestSensorUpdate ( Sensor::SensorID sensor )
{
#ifdef __AVR__
    controlPrivate = m_p;
#endif

    write ( Sensors );
    write ( sensor );

#ifdef __AVR__
    m_p->waitForSensors ( sensor );
#else
    uint8_t sensors[Sensor::packetSize ( sensor ) ];
    for ( int i = 0;i < Sensor::packetSize ( sensor );i++ ) {
        Conexus::Data d;
        try {
            d = m_p->m_device->read ( sizeof ( uint8_t ), -1 );
        } catch ( Conexus::exception::read::timeout e ) {
            m_p->m_robot->disconnected();
            return;
        }
        m_p->m_robot->m_sensorValues[i+Sensor::packetOffset ( sensor ) ] = * ( d.data() );
    }
    switch ( sensor ) {
        case Sensor::Group0:
        case Sensor::Group1:
        case Sensor::Group2:
        case Sensor::Group3:
        case Sensor::Group4:
        case Sensor::Group5:
        case Sensor::All:
            for ( int i = 7;i<=42;i++ ) {
                m_p->m_robot->sensorUpdated ( (Sensor::SensorID ) ( i ) );
            }
            break;
        default:
            m_p->m_robot->sensorUpdated ( sensor );
    }
#endif
}

