#ifndef SERIALCONTROL_H
#define SERIALCONTROL_H

#include <libirobot/Sensor.h>
#include <libirobot/Song.h>

// Baud UBRRx values
#define Ubrr300         3839
#define Ubrr600         1919
#define Ubrr1200        959
#define Ubrr2400        479
#define Ubrr4800        239
#define Ubrr9600        119
#define Ubrr14400       79
#define Ubrr19200       59
#define Ubrr28800       39
#define Ubrr38400       29
#define Ubrr57600       19
#define Ubrr115200      9

namespace iRobot
{

class Robot;

namespace Private {
class SerialControl;
}

/**
 * Interface to the serial-based Open Interface for the iRobot set of robots.
 *
 * The SerialControl class encaspulates the serial communication with the Open
 * Interface so that you don't have to know if you're on the control module's
 * atmega126 or running off of a laptop riding on top (make sure it is secured
 * down!)
 *
 * @see Robot::getSerial()
 */
class SerialControl
{
    public:
		/**
		 * Opcodes for the Open Interface.
		 */
        typedef enum {
            Start = 128,
            SetBaud = 129,
            Control = 130,
            Safe = 131,
            Full = 132,
            Demo = 136,
            Cover = 135,
            CoverAndDock = 143,
            Spot = 134,
            Drive = 137,
            DriveDirect = 145,
            LED = 139,
            DigitalOut = 147,
            PWMLowDrive = 144,
            SetLowDrive = 138,
            SendIR = 151,
            SaveSong = 140,
            PlaySong = 141,
            QueryList = 149,
            Sensors = 142,
            Stream = 148,
            PauseStream = 150,
            SaveScript = 152,
            RunScript = 153,
            ShowScript = 154,
            Wait = 155,
            WaitDistance = 156,
            WaitAngle = 157,
            WaitEvent = 158
        } OpCode;

		/**
		 * Baud rates as defined in the Open Interface spec
		 */
        typedef enum {
            Baud300 = 0,
            Baud600 = 1,
            Baud1200 = 2,
            Baud2400 = 3,
            Baud4800 = 4,
            Baud9600 = 5,
            Baud14400 = 6,
            Baud19200 = 7,
            Baud28800 = 8,
            Baud38400 = 9,
            Baud57600 = 10,
            Baud115200 = 11
        } Baud;

		/**
		 * Returns the robot this SerialControl is connected to
		 */
        Robot* getRobot() const;

		/**
		 * Sends the raw byte across the wire
		 */
        void write ( uint8_t value );

		/**
		 * Sends the opcode across the wire
		 */
        void write ( OpCode op );

		/**
		 * Serializes and then writes a song.
		 * FIXME: Internal?
		 *
		 * @see Robot::setSong()
		 */
		void write ( const Song &song );

		/**
		 * Writes a song's note across the wire in the format of
		 * [pitch][duration].
		 * FIXME: Internal?
		 *
		 * @see Robot::setSong()
		 */
		void write ( const Song::Note &note );

		/**
		 * Sets the speed of the serial link
		 */
        void setBaud ( Baud baud );

		/**
		 * Sends a request for the given sensor or sensor group, waits for a
		 * response, and then updates the sensor(s)
		 *
		 * @see Robot::getSensor()
		 */
        void requestSensorUpdate ( Sensor::SensorID sensor );
    private:
        friend class Robot;
        friend class Private::SerialControl;

        SerialControl ( Robot* robot, const char ttyDevice[] );

        ~SerialControl();
        Private::SerialControl* m_p;
};

}

#endif
