#ifndef SONG_H
#define SONG_H

#include <libirobot/types.h>

namespace iRobot {

/**
 * Representation of a song
 *
 * A song is composed of a series of Notes. Each note has an associated pitch
 * and length. Pitches are according to the MIDI numbering scheme, and range
 * from 31-127. Anything outside of that range is considered a rest.
 *
 * Note lengths are given in units of 1/64th of a second. Eg a half-second note
 * has a length of 32.
 *
 * @see Robot::playSong()
 * @see Robot::setSong()
 */
class Song {
	public:
		/**
		 * A note in a song.
		 * @see Song
		 */
		typedef struct {
			/**
			 * Pitch of the note, according to the MIDI numbering scheme. Can
			 * range from 31 to 127.
			 */
			uint8_t pitch;
			/**
			 * Length of the note, in units of 1/64th of a second.
			 */
			uint8_t length;
		} Note;

		/**
		 * Constructor for a blank song.
		 */
		Song();

		/**
		 * Constructor for a blank song of a given length
		 * @param length The number of notes in the song
		 */
		explicit Song(uint8_t length);

		/**
		 * Constructor for a song of a given length with the given notes
		 * @param length The number of notes in the song.
		 * @param notes The notes in the song
		 */
		Song(uint8_t length, const Note notes[]);

		/**
		 * Destructor.
		 */
		~Song();

		/**
		 * Sets the length of the song.
		 *
		 * The notes in the song are unchanged. If length is shorter than the
		 * current length, the song is truncated. Blank notes are appended if
		 * the song grows in length.
		 * 
		 * @param length New length of the song
		 */
		void setLength(uint8_t length);

		/**
		 * Changes the song to the given length and the given set of notes
		 *
		 * @param length Length of song
		 * @param notes Notes in song
		 */
		void setNotes(uint8_t length, const Note notes[]);

		/**
		 * Sets pitch of the the nth note of the song.
		 */
		void setNotePitch(int note, uint8_t pitch);

		/**
		 * Sets the length of the nth note of the song
		 */
		void setNoteLength(int note, uint8_t length);

		/**
		 * Sets the note at the given position
		 */
		void setNote(int noteNum, const Note note);

		/**
		 * Gets the note at the given position
		 */
		Note getNote(int note) const;
		
		/**
		 * Returns the length of the song in notes
		 */
		uint8_t getLength() const;
	private:
		uint8_t m_length;
		Note* m_notes;
};

}

#endif

